#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007 John Bintz, jcoswell@cosellproductions.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Greate perfect bound cover
"""
import textwrap

import inkex
from inkex import Guide


def caliper_to_ppi(caliper):
    return 2 / caliper


def bond_weight_to_ppi(bond_weight):
    return caliper_to_ppi(bond_weight * 0.0002)


def points_to_ppi(points):
    return caliper_to_ppi(points / 1000.0)


class PerfectBoundCover(inkex.EffectExtension):
    def add_arguments(self, pars):
        pars.add_argument("--width", type=float, default=6.0, help="cover width (in)")
        pars.add_argument("--height", type=float, default=9.0, help="cover height (in)")
        pars.add_argument("--pages", type=int, default=64, help="number of pages")
        pars.add_argument(
            "--paperthicknessmeasurement",
            default="ppi",
            help="""Measurement for determining the thickness of the spine.
                          Options: 'ppi': pages per inch; 'caliper': caliper (inches);
                            'points': caliper in points (1/1000 in);
                            'bond_weight': Bond (pounds);
                            'width': absolute width of spine (in)""",
        )
        pars.add_argument(
            "--paperthickness", type=float, default=0.0, help="paper thickness"
        )
        pars.add_argument(
            "--coverthicknessmeasurement",
            default="ppi",
            help="Measurement for determining the thickness of the cover. For "
            "available options, see --paperthicknessmeasurement",
        )
        pars.add_argument(
            "--coverthickness", type=float, default=0.0, help="cover thickness"
        )
        pars.add_argument("--bleed", type=float, default=0.25, help="cover bleed (in)")
        pars.add_argument(
            "--removeguides", type=inkex.Boolean, default=False, help="remove guide"
        )

    def effect(self):
        switch = {
            "ppi": lambda x: x,
            "caliper": caliper_to_ppi,
            "bond_weight": bond_weight_to_ppi,
            "points": points_to_ppi,
            "width": lambda x: x,
        }

        if self.options.paperthickness > 0:
            if self.options.paperthicknessmeasurement == "width":
                paper_spine = self.options.paperthickness
            else:
                paper_spine = self.options.pages / switch[
                    self.options.paperthicknessmeasurement
                ](self.options.paperthickness)
        else:
            paper_spine = 0

        if self.options.coverthickness > 0:
            if self.options.coverthicknessmeasurement == "width":
                cover_spine = self.options.coverthickness
            else:
                cover_spine = 4.0 / switch[self.options.coverthicknessmeasurement](
                    self.options.coverthickness
                )
        else:
            cover_spine = 0

        spine_width = paper_spine + cover_spine

        document_width = (self.options.bleed + self.options.width * 2) + spine_width
        document_height = self.options.bleed * 2 + self.options.height

        root = self.document.getroot()

        root.set("width", "%sin" % document_width)
        root.set("height", "%sin" % document_height)

        guides = []

        guides.append(["horizontal", self.options.bleed])
        guides.append(["horizontal", document_height - self.options.bleed])
        guides.append(["vertical", self.options.bleed])
        guides.append(["vertical", document_width - self.options.bleed])
        guides.append(["vertical", (document_width / 2) - (spine_width / 2)])
        guides.append(["vertical", (document_width / 2) + (spine_width / 2)])

        namedview = self.svg.namedview
        if namedview is not None:
            if self.options.removeguides:
                for node in namedview.get_guides():
                    node.delete()
            for guide in guides:
                newguide = namedview.add(Guide())
                newguide.set("orientation", guide[0])
                newguide.set("position", "%f" % (guide[1] * 96))


if __name__ == "__main__":
    PerfectBoundCover().run()



#!/usr/bin/env python
#
# Copyright (C) 2007 Aaron Spike  (aaron @ ekips.org)
# Copyright (C) 2007 Tavmjong Bah (tavmjong @ free.fr)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Generate gears in SVG
"""

from math import acos, cos, pi, radians, sin, sqrt

import inkex
from inkex import PathElement


def involute_intersect_angle(Rb, R):
    Rb, R = float(Rb), float(R)
    return (sqrt(R**2 - Rb**2) / Rb) - (acos(Rb / R))


def point_on_circle(radius, angle):
    x = radius * cos(angle)
    y = radius * sin(angle)
    return x, y


def points_to_svgd(p):
    f = p[0]
    p = p[1:]
    svgd = "M{:.5f},{:.5f}".format(f[0], f[1])
    for x in p:
        svgd += " L{:.5f},{:.5f}".format(x[0], x[1])
    svgd += "z"
    return svgd


class Gears(inkex.GenerateExtension):
    container_label = "Rendered Gears"

    def add_arguments(self, pars):
        pars.add_argument("--teeth", type=int, default=24, help="Number of teeth")
        pars.add_argument("--pitch", type=float, default=20.0, help="Circular Pitch")
        pars.add_argument("--angle", type=float, default=20.0, help="Pressure Angle")
        pars.add_argument(
            "--centerdiameter", type=float, default=20.0, help="Diameter of hole"
        )
        pars.add_argument(
            "--unit", default="px", help="unit for pitch and center diameter"
        )

    def generate(self):
        teeth = self.options.teeth
        pitch = self.svg.unittouu(str(self.options.pitch) + self.options.unit)
        angle = (
            self.options.angle
        )  # Angle of tangent to tooth at circular pitch wrt radial line.
        centerdiameter = self.svg.unittouu(
            str(self.options.centerdiameter) + self.options.unit
        )

        # print >>sys.stderr, "Teeth: %s\n"        % teeth

        two_pi = 2.0 * pi

        # Pitch (circular pitch): Length of the arc from one tooth to the next)
        # Pitch diameter: Diameter of pitch circle.
        pitch_diameter = float(teeth) * pitch / pi
        pitch_radius = pitch_diameter / 2.0

        # Base Circle
        base_diameter = pitch_diameter * cos(radians(angle))
        base_radius = base_diameter / 2.0

        # Diametrial pitch: Number of teeth per unit length.
        pitch_diametrial = float(teeth) / pitch_diameter

        # Addendum: Radial distance from pitch circle to outside circle.
        addendum = 1.0 / pitch_diametrial

        # Outer Circle
        outer_radius = pitch_radius + addendum
        outer_diameter = outer_radius * 2.0

        # Tooth thickness: Tooth width along pitch circle.
        tooth = (pi * pitch_diameter) / (2.0 * float(teeth))

        # Undercut?
        undercut = 2.0 / (sin(radians(angle)) ** 2)
        needs_undercut = teeth < undercut

        # Clearance: Radial distance between top of tooth on one gear to bottom of gap on another.
        clearance = 0.0

        # Dedendum: Radial distance from pitch circle to root diameter.
        dedendum = addendum + clearance

        # Root diameter: Diameter of bottom of tooth spaces.
        root_radius = pitch_radius - dedendum
        root_diameter = root_radius * 2.0

        half_thick_angle = two_pi / (4.0 * float(teeth))
        pitch_to_base_angle = involute_intersect_angle(base_radius, pitch_radius)
        pitch_to_outer_angle = (
            involute_intersect_angle(base_radius, outer_radius) - pitch_to_base_angle
        )

        centers = [(x * two_pi / float(teeth)) for x in range(teeth)]

        points = []

        for c in centers:

            # Angles
            pitch1 = c - half_thick_angle
            base1 = pitch1 - pitch_to_base_angle
            outer1 = pitch1 + pitch_to_outer_angle

            pitch2 = c + half_thick_angle
            base2 = pitch2 + pitch_to_base_angle
            outer2 = pitch2 - pitch_to_outer_angle

            # Points
            b1 = point_on_circle(base_radius, base1)
            p1 = point_on_circle(pitch_radius, pitch1)
            o1 = point_on_circle(outer_radius, outer1)

            b2 = point_on_circle(base_radius, base2)
            p2 = point_on_circle(pitch_radius, pitch2)
            o2 = point_on_circle(outer_radius, outer2)

            if root_radius > base_radius:
                pitch_to_root_angle = pitch_to_base_angle - involute_intersect_angle(
                    base_radius, root_radius
                )
                root1 = pitch1 - pitch_to_root_angle
                root2 = pitch2 + pitch_to_root_angle
                r1 = point_on_circle(root_radius, root1)
                r2 = point_on_circle(root_radius, root2)
                p_tmp = [r1, p1, o1, o2, p2, r2]
            else:
                r1 = point_on_circle(root_radius, base1)
                r2 = point_on_circle(root_radius, base2)
                p_tmp = [r1, b1, p1, o1, o2, p2, b2, r2]

            points.extend(p_tmp)

        path = points_to_svgd(points)

        # Create SVG Path for gear
        style = {
            "stroke": "#000000",
            "fill": "none",
            "stroke-width": str(self.svg.unittouu("1px")),
        }
        gear = PathElement()
        gear.style = style
        gear.path = path
        yield gear

        if centerdiameter > 0.0:
            arc = PathElement.arc((0, 0), centerdiameter / 2)
            arc.style = style
            yield arc


if __name__ == "__main__":
    Gears().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007 John Beard john.j.beard@gmail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
This extension allows you to draw various triangle constructions
It requires a path to be selected
It will use the first three nodes of this path

Dimensions of a triangle__

       /`__
      / a_c``--__
     /           ``--__ s_a
s_b /                  ``--__
   /a_a                    a_b`--__
  /--------------------------------``B
 A              s_b
"""

from math import acos, cos, pi, sin, sqrt, tan

import inkex
from inkex import PathElement, Circle

(X, Y) = range(2)

# DRAWING ROUTINES

# draw an SVG triangle given in trilinar coords
def draw_SVG_circle(
    rad, centre, params, style, name, parent
):  # draw an SVG circle with a given radius as trilinear coordinates
    if rad == 0:  # we want a dot
        r = style.d_rad  # get the dot width from the style
        circ_style = {
            "stroke": style.d_col,
            "stroke-width": str(style.d_th),
            "fill": style.d_fill,
        }
    else:
        r = rad  # use given value
        circ_style = {
            "stroke": style.c_col,
            "stroke-width": str(style.c_th),
            "fill": style.c_fill,
        }

    cx, cy = get_cartesian_pt(centre, params)
    circ_attribs = {"cx": str(cx), "cy": str(cy), "r": str(r)}
    elem = parent.add(Circle(**circ_attribs))
    elem.style = circ_style
    elem.label = name


# draw an SVG triangle given in trilinar coords
def draw_SVG_tri(vert_mat, params, style, name, parent):
    p1, p2, p3 = get_cartesian_tri(
        vert_mat, params
    )  # get the vertex matrix in cartesian points
    elem = parent.add(PathElement())
    elem.path = (
        "M "
        + str(p1[0])
        + ","
        + str(p1[1])
        + " L "
        + str(p2[0])
        + ","
        + str(p2[1])
        + " L "
        + str(p3[0])
        + ","
        + str(p3[1])
        + " L "
        + str(p1[0])
        + ","
        + str(p1[1])
        + " z"
    )
    elem.style = {
        "stroke": style.l_col,
        "stroke-width": str(style.l_th),
        "fill": style.l_fill,
    }
    elem.label = name


# draw an SVG line segment between the given (raw) points
def draw_SVG_line(a, b, style, name, parent):
    (x1, y1) = a
    (x2, y2) = b
    line = parent.add(PathElement())
    line.style = {
        "stroke": style.l_col,
        "stroke-width": str(style.l_th),
        "fill": style.l_fill,
    }
    line.path = "M " + str(x1) + "," + str(y1) + " L " + str(x2) + "," + str(y2)
    line.lavel = name


# lines from each vertex to a corresponding point in trilinears
def draw_vertex_lines(vert_mat, params, width, name, parent):
    for i in range(3):
        oppositepoint = get_cartesian_pt(vert_mat[i], params)
        draw_SVG_line(
            params[3][-i % 3], oppositepoint, width, name + ":" + str(i), parent
        )


# MATHEMATICAL ROUTINES


def distance(a, b):
    """find the pythagorean distance"""
    (x0, y0) = a
    (x1, y1) = b
    return sqrt((x0 - x1) * (x0 - x1) + (y0 - y1) * (y0 - y1))


def vector_from_to(a, b):
    """get the vector from (x0,y0) to (x1,y1)"""
    return b[X] - a[X], b[Y], a[Y]


def get_cartesian_pt(t, p):  # get the cartesian coordinates from a trilinear set
    denom = p[0][0] * t[0] + p[0][1] * t[1] + p[0][2] * t[2]
    c1 = p[0][1] * t[1] / denom
    c2 = p[0][2] * t[2] / denom
    return c1 * p[2][1][0] + c2 * p[2][0][0], c1 * p[2][1][1] + c2 * p[2][0][1]


def get_cartesian_tri(arg, params):
    """get the cartesian points from a trilinear vertex matrix"""
    (t11, t12, t13), (t21, t22, t23), (t31, t32, t33) = arg
    p1 = get_cartesian_pt((t11, t12, t13), params)
    p2 = get_cartesian_pt((t21, t22, t23), params)
    p3 = get_cartesian_pt((t31, t32, t33), params)
    return p1, p2, p3


def angle_from_3_sides(a, b, c):  # return the angle opposite side c
    cosx = (a * a + b * b - c * c) / (2 * a * b)  # use the cosine rule
    return acos(cosx)


def translate_string(
    string, os
):  # translates s_a, a_a, etc to params[x][y], with cyclic offset
    string = string.replace(
        "s_a", "params[0][" + str((os + 0) % 3) + "]"
    )  # replace with ref. to the relvant values,
    string = string.replace(
        "s_b", "params[0][" + str((os + 1) % 3) + "]"
    )  # cycled by i
    string = string.replace("s_c", "params[0][" + str((os + 2) % 3) + "]")
    string = string.replace("a_a", "params[1][" + str((os + 0) % 3) + "]")
    string = string.replace("a_b", "params[1][" + str((os + 1) % 3) + "]")
    string = string.replace("a_c", "params[1][" + str((os + 2) % 3) + "]")
    string = string.replace("area", "params[4][0]")
    string = string.replace("semiperim", "params[4][1]")
    return string


def pt_from_tcf(
    tcf, params
):  # returns a trilinear triplet from a triangle centre function
    trilin_pts = []  # will hold the final points
    for i in range(3):
        temp = tcf  # read in the tcf
        temp = translate_string(temp, i)
        func = eval(
            "lambda params: " + temp.strip('"')
        )  # the function leading to the trilinar element
        trilin_pts.append(
            func(params)
        )  # evaluate the function for the first trilinear element
    return trilin_pts


# SVG DATA PROCESSING


def get_n_points_from_path(node, n):
    """returns a list of first n points (x,y) in an SVG path-representing node"""
    points = list(node.path.control_points)
    if len(points) < 3:
        return []
    return points[:3]


# EXTRA MATHS FUNCTIONS
def sec(x):  # secant(x)
    if (
        x == pi / 2 or x == -pi / 2 or x == 3 * pi / 2 or x == -3 * pi / 2
    ):  # sec(x) is undefined
        return 100000000000
    else:
        return 1 / cos(x)


def csc(x):  # cosecant(x)
    if x == 0 or x == pi or x == 2 * pi or x == -2 * pi:  # csc(x) is undefined
        return 100000000000
    else:
        return 1 / sin(x)


def cot(x):  # cotangent(x)
    if x == 0 or x == pi or x == 2 * pi or x == -2 * pi:  # cot(x) is undefined
        return 100000000000
    else:
        return 1 / tan(x)


class Style(object):  # container for style information
    def __init__(self, svg):
        # dot markers
        self.d_rad = svg.unittouu("4px")  # dot marker radius
        self.d_th = svg.unittouu("2px")  # stroke width
        self.d_fill = "#aaaaaa"  # fill colour
        self.d_col = "#000000"  # stroke colour

        # lines
        self.l_th = svg.unittouu("2px")
        self.l_fill = "none"
        self.l_col = "#000000"

        # circles
        self.c_th = svg.unittouu("2px")
        self.c_fill = "none"
        self.c_col = "#000000"


class DrawFromTriangle(inkex.EffectExtension):
    def add_arguments(self, pars):
        pars.add_argument("--tab")
        # PRESET POINT OPTIONS
        pars.add_argument("--circumcircle", type=inkex.Boolean, default=False)
        pars.add_argument("--circumcentre", type=inkex.Boolean, default=False)
        pars.add_argument("--incircle", type=inkex.Boolean, default=False)
        pars.add_argument("--incentre", type=inkex.Boolean, default=False)
        pars.add_argument("--contact_tri", type=inkex.Boolean, default=False)
        pars.add_argument("--excircles", type=inkex.Boolean, default=False)
        pars.add_argument("--excentres", type=inkex.Boolean, default=False)
        pars.add_argument("--extouch_tri", type=inkex.Boolean, default=False)
        pars.add_argument("--excentral_tri", type=inkex.Boolean, default=False)
        pars.add_argument("--orthocentre", type=inkex.Boolean, default=False)
        pars.add_argument("--orthic_tri", type=inkex.Boolean, default=False)
        pars.add_argument("--altitudes", type=inkex.Boolean, default=False)
        pars.add_argument("--anglebisectors", type=inkex.Boolean, default=False)
        pars.add_argument("--centroid", type=inkex.Boolean, default=False)
        pars.add_argument("--ninepointcentre", type=inkex.Boolean, default=False)
        pars.add_argument("--ninepointcircle", type=inkex.Boolean, default=False)
        pars.add_argument("--symmedians", type=inkex.Boolean, default=False)
        pars.add_argument("--sym_point", type=inkex.Boolean, default=False)
        pars.add_argument("--sym_tri", type=inkex.Boolean, default=False)
        pars.add_argument("--gergonne_pt", type=inkex.Boolean, default=False)
        pars.add_argument("--nagel_pt", type=inkex.Boolean, default=False)
        # CUSTOM POINT OPTIONS
        pars.add_argument("--mode", default="trilin")
        pars.add_argument("--cust_str", default="cos(a_a):cos(a_b):cos(a_c)")
        pars.add_argument("--cust_pt", type=inkex.Boolean, default=False)
        pars.add_argument("--cust_radius", type=inkex.Boolean, default=False)
        pars.add_argument("--radius", default="s_a*s_b*s_c/(4*area)")
        pars.add_argument("--isogonal_conj", type=inkex.Boolean, default=False)
        pars.add_argument("--isotomic_conj", type=inkex.Boolean, default=False)

    def effect(self):
        so = self.options  # shorthand

        pts = (
            []
        )  # initialise in case nothing is selected and following loop is not executed
        for node in self.svg.selection.filter(inkex.PathElement):
            # find the (x,y) coordinates of the first 3 points of the path
            pts = get_n_points_from_path(node, 3)

        if len(pts) == 3:  # if we have right number of nodes, else skip and end program
            st = Style(self.svg)  # style for dots, lines and circles

            # CREATE A GROUP TO HOLD ALL GENERATED ELEMENTS IN
            # Hold relative to point A (pt[0])
            layer = self.svg.get_current_layer().add(
                inkex.Group.new("TriangleElements")
            )
            layer.transform = "translate(" + str(pts[0][0]) + "," + str(pts[0][1]) + ")"

            # GET METRICS OF THE TRIANGLE
            # vertices in the local coordinates (set pt[0] to be the origin)
            vtx = [
                [0, 0],
                [pts[1][0] - pts[0][0], pts[1][1] - pts[0][1]],
                [pts[2][0] - pts[0][0], pts[2][1] - pts[0][1]],
            ]

            s_a = distance(vtx[1], vtx[2])  # get the scalar side lengths
            s_b = distance(vtx[0], vtx[1])
            s_c = distance(vtx[0], vtx[2])
            sides = (
                s_a,
                s_b,
                s_c,
            )  # side list for passing to functions easily and for indexing

            a_a = angle_from_3_sides(s_b, s_c, s_a)  # angles in radians
            a_b = angle_from_3_sides(s_a, s_c, s_b)
            a_c = angle_from_3_sides(s_a, s_b, s_c)
            angles = (a_a, a_b, a_c)

            ab = vector_from_to(vtx[0], vtx[1])  # vector from a to b
            ac = vector_from_to(vtx[0], vtx[2])  # vector from a to c
            bc = vector_from_to(vtx[1], vtx[2])  # vector from b to c
            vecs = (ab, ac)  # vectors for finding cartesian point from trilinears

            semiperim = (s_a + s_b + s_c) / 2.0  # semiperimeter
            area = sqrt(
                semiperim * (semiperim - s_a) * (semiperim - s_b) * (semiperim - s_c)
            )  # area of the triangle by heron's formula
            uvals = (area, semiperim)  # useful values

            params = (
                sides,
                angles,
                vecs,
                vtx,
                uvals,
            )  # all useful triangle parameters in one object

            # BEGIN DRAWING
            if so.circumcentre or so.circumcircle:
                r = s_a * s_b * s_c / (4 * area)
                pt = (cos(a_a), cos(a_b), cos(a_c))
                if so.circumcentre:
                    draw_SVG_circle(0, pt, params, st, "Circumcentre", layer)
                if so.circumcircle:
                    draw_SVG_circle(r, pt, params, st, "Circumcircle", layer)

            if so.incentre or so.incircle:
                pt = [1, 1, 1]
                if so.incentre:
                    draw_SVG_circle(0, pt, params, st, "Incentre", layer)
                if so.incircle:
                    r = area / semiperim
                    draw_SVG_circle(r, pt, params, st, "Incircle", layer)

            if so.contact_tri:
                t1 = s_b * s_c / (-s_a + s_b + s_c)
                t2 = s_a * s_c / (s_a - s_b + s_c)
                t3 = s_a * s_b / (s_a + s_b - s_c)
                v_mat = ((0, t2, t3), (t1, 0, t3), (t1, t2, 0))
                draw_SVG_tri(v_mat, params, st, "ContactTriangle", layer)

            if so.extouch_tri:
                t1 = (-s_a + s_b + s_c) / s_a
                t2 = (s_a - s_b + s_c) / s_b
                t3 = (s_a + s_b - s_c) / s_c
                v_mat = ((0, t2, t3), (t1, 0, t3), (t1, t2, 0))
                draw_SVG_tri(v_mat, params, st, "ExtouchTriangle", layer)

            if so.orthocentre:
                pt = pt_from_tcf("cos(a_b)*cos(a_c)", params)
                draw_SVG_circle(0, pt, params, st, "Orthocentre", layer)

            if so.orthic_tri:
                v_mat = [
                    [0, sec(a_b), sec(a_c)],
                    [sec(a_a), 0, sec(a_c)],
                    [sec(a_a), sec(a_b), 0],
                ]
                draw_SVG_tri(v_mat, params, st, "OrthicTriangle", layer)

            if so.centroid:
                pt = [1 / s_a, 1 / s_b, 1 / s_c]
                draw_SVG_circle(0, pt, params, st, "Centroid", layer)

            if so.ninepointcentre or so.ninepointcircle:
                pt = [cos(a_b - a_c), cos(a_c - a_a), cos(a_a - a_b)]
                if so.ninepointcentre:
                    draw_SVG_circle(0, pt, params, st, "NinePointCentre", layer)
                if so.ninepointcircle:
                    r = s_a * s_b * s_c / (8 * area)
                    draw_SVG_circle(r, pt, params, st, "NinePointCircle", layer)

            if so.altitudes:
                v_mat = [
                    [0, sec(a_b), sec(a_c)],
                    [sec(a_a), 0, sec(a_c)],
                    [sec(a_a), sec(a_b), 0],
                ]
                draw_vertex_lines(v_mat, params, st, "Altitude", layer)

            if so.anglebisectors:
                v_mat = ((0, 1, 1), (1, 0, 1), (1, 1, 0))
                draw_vertex_lines(v_mat, params, st, "AngleBisectors", layer)

            if so.excircles or so.excentres or so.excentral_tri:
                v_mat = ((-1, 1, 1), (1, -1, 1), (1, 1, -1))
                if so.excentral_tri:
                    draw_SVG_tri(v_mat, params, st, "ExcentralTriangle", layer)
                for i in range(3):
                    if so.excircles:
                        r = area / (semiperim - sides[i])
                        draw_SVG_circle(
                            r, v_mat[i], params, st, "Excircle:" + str(i), layer
                        )
                    if so.excentres:
                        draw_SVG_circle(
                            0, v_mat[i], params, st, "Excentre:" + str(i), layer
                        )

            if so.sym_tri or so.symmedians:
                v_mat = ((0, s_b, s_c), (s_a, 0, s_c), (s_a, s_b, 0))
                if so.sym_tri:
                    draw_SVG_tri(v_mat, params, st, "SymmedialTriangle", layer)
                if so.symmedians:
                    draw_vertex_lines(v_mat, params, st, "Symmedian", layer)

            if so.sym_point:
                pt = (s_a, s_b, s_c)
                draw_SVG_circle(0, pt, params, st, "SymmmedianPoint", layer)

            if so.gergonne_pt:
                pt = pt_from_tcf("1/(s_a*(s_b+s_c-s_a))", params)
                draw_SVG_circle(0, pt, params, st, "GergonnePoint", layer)

            if so.nagel_pt:
                pt = pt_from_tcf("(s_b+s_c-s_a)/s_a", params)
                draw_SVG_circle(0, pt, params, st, "NagelPoint", layer)

            if so.cust_pt or so.cust_radius or so.isogonal_conj or so.isotomic_conj:
                pt = []  # where we will store the point in trilinears
                if so.mode == "trilin":  # if we are receiving from trilinears
                    for i in range(3):
                        strings = so.cust_str.split(":")  # get split string
                        strings[i] = translate_string(strings[i], 0)
                        func = eval(
                            "lambda params: " + strings[i].strip('"')
                        )  # the function leading to the trilinar element
                        pt.append(
                            func(params)
                        )  # evaluate the function for the trilinear element
                else:  # we need a triangle function
                    string = (
                        so.cust_str
                    )  # don't need to translate, as the pt_from_tcf function does that for us
                    pt = pt_from_tcf(
                        string, params
                    )  # get the point from the tcf directly

                if so.cust_pt:  # draw the point
                    draw_SVG_circle(0, pt, params, st, "CustomTrilinearPoint", layer)
                if so.cust_radius:  # draw the circle with given radius
                    strings = translate_string(so.radius, 0)
                    func = eval(
                        "lambda params: " + strings.strip('"')
                    )  # the function leading to the radius
                    r = func(params)
                    draw_SVG_circle(r, pt, params, st, "CustomTrilinearCircle", layer)
                if so.isogonal_conj:
                    isogonal = [0, 0, 0]
                    for i in range(3):
                        isogonal[i] = 1 / pt[i]
                    draw_SVG_circle(
                        0, isogonal, params, st, "CustomIsogonalConjugate", layer
                    )
                if so.isotomic_conj:
                    isotomic = [0, 0, 0]
                    for i in range(3):
                        isotomic[i] = 1 / (params[0][i] * params[0][i] * pt[i])
                    draw_SVG_circle(
                        0, isotomic, params, st, "CustomIsotomicConjugate", layer
                    )


if __name__ == "__main__":
    DrawFromTriangle().run()



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Install jessyInk scripts"""

import inkex
from inkex import Script

from inkex.localization import inkex_gettext as _

inkex.NSS["jessyink"] = "https://launchpad.net/jessyink"


class JessyInkMixin(object):
    """Common jessyInk items"""

    def is_installed(self):
        """Check jessyInk is installed correctly"""
        scripts = self.svg.getElement("//svg:script[@jessyink:version='1.5.5']")
        if scripts is None:
            raise inkex.AbortExtension(
                _(
                    "The JessyInk script is not installed in this SVG file or has a "
                    "different version than the JessyInk extensions. Please select "
                    '"install/update..." from the "JessyInk" sub-menu of the "Extensions" '
                    "menu to install or update the JessyInk script.\n\n"
                )
            )

    def attr_remove(self, prop, is_removed=True):
        """Remove a property if it exists in the svg"""
        if is_removed:
            for node in self.svg.xpath(f"//*[@jessyink:{prop}]"):
                node.set(f"jessyink:{prop}", None)
            for node in self.svg.xpath(f"//*[@jessyInk_{prop}]"):
                node.set(f"jessyInk_{prop}", None)

    def attr_update(self, name):
        """Update a single attr"""
        for node in self.svg.xpath(f"//*[@jessyInk_{name}]"):
            node.set(f"jessyink:{name}", node.get(f"jessyInk_{name}"))
            node.set(f"jessyInk_{name}", None)
        for node in self.svg.xpath(f"//*[@jessyink:{name}]"):
            node.set(f"jessyink:{name}", node.get(f"jessyink:{name}").replace("=", ":"))

    @staticmethod
    def prop_str_to_list(string):
        """Script string to list of instructions"""
        return [prop.strip() for prop in string.split(";") if prop]

    @staticmethod
    def list_to_prop_str(lst):
        """List of instructions to script string"""
        return "; ".join(lst) + ";"


class Install(JessyInkMixin, inkex.EffectExtension):
    """Install jessyInk extension into an SVG"""

    def add_arguments(self, pars):
        pars.add_argument("--tab", type=str, dest="what")

    def effect(self):
        # Find and delete old script node
        for node in self.svg.xpath("//svg:script[@id='JessyInk']"):
            node.getparent().remove(node)

        # Create new script node
        script_elem = Script()
        with open(self.get_resource("jessyInk.js")) as fhl:
            script_elem.text = fhl.read()
        script_elem.set("id", "JessyInk")
        script_elem.set("jessyink:version", "1.5.5")
        self.svg.append(script_elem)

        # Remove "jessyInkInit()" in the "onload" attribute, if present.
        prop_list = [prop.strip() for prop in self.svg.get("onload", "").split(";")]
        if "jessyInkInit()" in prop_list:
            prop_list.remove("jessyInkInit()")
        self.svg.set("onload", "; ".join(prop_list) or None)

        # Update jessyInk attributes to new formats
        for attr in (
            "effectIn",
            "effectOut",
            "masterSlide",
            "transitionIn",
            "transitionOut",
            "autoText",
        ):
            self.attr_update(attr)


# Create effect instance
if __name__ == "__main__":
    Install().run()



# coding=utf-8
#
# Copyright 2022 Martin Owens <doctormo@geek-2.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Structures for consistant testing of Gtk GUI programs.
"""

import sys
from gi.repository import Gtk, GLib


class MainLoopProtection:
    """
    This protection class provides a way to launch the Gtk mainloop in a test
    friendly way.

    Exception handling hooks provide a way to see errors that happen
        inside the main loop, raising them back to the caller.
    A full timeout in seconds stops the gtk mainloop from operating
        beyond a set time, acting as a kill switch in the event something
        has gone horribly wrong.

    Use:
        with MainLoopProtection(timeout=10s):
            app.run()
    """

    def __init__(self, timeout=10):
        self.timeout = timeout * 1000
        self._hooked = None
        self._old_excepthook = None

    def __enter__(self):
        # replace sys.excepthook with our own and remember hooked raised error
        self._old_excepthook = sys.excepthook
        sys.excepthook = self.excepthook
        # Remove mainloop by force if it doesn't die within 10 seconds
        self._timeout = GLib.timeout_add(self.timeout, self.idle_exit)

    def __exit__(self, exc, value, traceback):  # pragma: no cover
        """Put the except handler back, cancel the timer and raise if needed"""
        if self._old_excepthook:
            sys.excepthook = self._old_excepthook
        # Remove the timeout, so we don't accidentally kill later mainloops
        if self._timeout:
            GLib.source_remove(self._timeout)
        # Raise an exception if one happened during the test run
        if self._hooked:
            exc, value, traceback = self._hooked
        if value and traceback:
            raise value.with_traceback(traceback)

    def idle_exit(self):  # pragma: no cover
        """Try to going to kill any running mainloop."""
        GLib.idle_add(Gtk.main_quit)

    def excepthook(self, ex_type, ex_value, traceback):  # pragma: no cover
        """Catch errors thrown by the Gtk mainloop"""
        self.idle_exit()
        # Remember the exception data for raising inside the test context
        if ex_value is not None:
            self._hooked = [ex_type, ex_value, traceback]
        # Fallback and double print the exception (remove if double printing is problematic)
        return self._old_excepthook(ex_type, ex_value, traceback)



#!/usr/bin/env python
"""Reduce saturation"""

import inkex


class LessSaturation(inkex.ColorExtension):
    """Make colours less saturated"""

    def modify_color(self, name, color):
        color.saturation -= int(0.05 * 255)
        return color


if __name__ == "__main__":
    LessSaturation().run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2021 Martin Owens <doctormo@gmail.com>
# Copyright 2022 Simon Duerr <dev@simonduerr.eu>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Import images from the internet, inkscape extension (GUI)
"""

__version__ = '1.0'
__pkgname__ = 'inkscape-import-web-image'

import os
import sys
import logging
import warnings
warnings.filterwarnings("ignore")

from collections import defaultdict
from base64 import encodebytes
from appdirs import user_cache_dir

import inkex
from inkex import Style
from inkex.gui import GtkApp, Window, IconView, asyncme
from inkex.gui.pixmap import PixmapManager, SizeFilter, PadFilter, OverlayFilter
from inkex.elements import (
    load_svg, Image, Defs, NamedView, Metadata,
    SvgDocumentElement, StyleElement
)
from gi.repository import Gtk

from import_sources import RemoteSource, RemoteFile, RemotePage

SOURCES = os.path.join(os.path.dirname(__file__), 'sources')
LICENSES = os.path.join(os.path.dirname(__file__), 'licenses')
CACHE_DIR = user_cache_dir('inkscape-import-web-image', 'Inkscape')

class LicenseOverlay(OverlayFilter):
    pixmaps = PixmapManager(LICENSES)

    def get_overlay(self, item=None, manager=None):
        if item is None: # Default image
            return None
        return self.pixmaps.get(item.get_overlay())


class ResultsIconView(IconView):
    """The search results shown as icons"""
    def get_markup(self, item):
        return item.string

    def get_icon(self, item):
        return item.icon

    def setup(self):
        self._list.set_markup_column(1)
        self._list.set_pixbuf_column(2)
        crt, crp = self._list.get_cells()
        self.crt_notify = crt.connect('notify', self.keep_size)
        super().setup()

    def keep_size(self, crt, *args):
        """Hack Gtk to keep cells smaller"""
        crt.handler_block(self.crt_notify)
        crt.set_property('width', 150)
        crt.handler_unblock(self.crt_notify)


class ImporterWindow(Window):
    """The window that is in the glade file"""
    name = 'import_web_image'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.widget('dl-searching').hide()

        # Add each of the source services from their plug-in modules
        self.source = self.widget('service_list')
        self.source_model = self.source.get_model()
        self.source_model.clear()

        RemoteSource.load(SOURCES)
        pix = PixmapManager(SOURCES, size=150)
        for x, (key, source) in enumerate(RemoteSource.sources.items()):
            if not source.is_enabled:
                continue
            # We add them in GdkPixbuf, string, string format (see glade file)
            self.source_model.append([pix.get(source.icon), source.name, key])
            if source.is_default:
                self.source.set_active(x)

        pixmaps = PixmapManager(CACHE_DIR, filters=[
            SizeFilter(size=150),
            PadFilter(size=(0, 150)),
            LicenseOverlay(position=(0.5, 1))
        ])
        self.select_func = self.gapp.kwargs['select']
        self.results = ResultsIconView(self.widget('results'), pixmaps)
        self.pool = ResultsIconView(self.widget('pool'), pixmaps)
        self.multiple = False

        self.widget("perm-nocopyright").set_message_type(Gtk.MessageType.WARNING)

    def select_image(self, widget):
        """Callback when clicking on an image in the result list"""
        for item_path in self.widget("results").get_selected_items():
            item_iter = self.results._model.get_iter(item_path)
            item = self.results._model[item_iter][0]
            self.show_license(item)
        self.update_btn_import()

    def show_license(self, item):
        """Display the current item's license information"""
        self.widget("perms").foreach(lambda w: w.hide())
        info = item.license_info
        for mod in info['modules']:
            self.widget("perm-" + mod).show()

    def get_selected_source(self):
        """Return the selected source class"""
        _iter = self.source.get_active_iter()
        key = self.source_model[_iter][2]
        return RemoteSource.sources[key](CACHE_DIR)

    def img_multiple(self, widget):
        """Enable multi-selection"""
        widget.hide()
        self.widget('btn-single').show()
        self.widget('multiple-box').show()
        self.widget('multiple-buttons').show()
        self.multiple = True
        self.update_btn_import()

    def img_single(self, widget):
        """Enable single-selection"""
        widget.hide()
        self.widget('btn-multiple').show()
        self.widget('multiple-box').hide()
        self.widget('multiple-buttons').hide()
        is_selected = bool(list(self.results.get_selected_items()))
        self.pool.clear()
        self.multiple = False
        self.update_btn_import()

    def img_add(self, widget=None):
        """Add an item from the search to the multi-pool"""
        self.pool.add(self.results.get_selected_items())
        self.update_btn_import()

    def img_remove(self, widget=None):
        """Remove any selected items from the multi-pool"""
        for item in self.pool.get_selected_items():
            self.pool.remove_item(item)
        self.update_btn_import()

    def update_btn_import(self):
        """Set the import button to enabled or disabled"""
        enabled = False
        if self.multiple:
            enabled = bool(list(self.pool))
        else:
            enabled = bool(list(self.results.get_selected_items()))
        self.widget('btn-import').set_sensitive(enabled)

    def result_activate(self, widget=None):
        """Search results double click"""
        if self.multiple:
            self.img_add()
        else:
            self.img_import()

    def img_import(self, widget=None):
        """Apply the selected image and quit"""
        items = []
        if not self.multiple:
            items.extend(self.results.get_selected_items())
        else:
            items.extend([item for item,*_ in self.pool])

        to_exit = True
        for item in items:
            self.select_func(item.get_file())
            # XXX This pagination control is not good. Replace it with normal controls.
            #elif isinstance(item, RemotePage):
        if to_exit:
            self.exit()

    def search(self, widget):
        """Remote search activation"""
        query = widget.get_text()
        if len(query) > 2:
            self.search_clear()
            self.search_started()
            self.async_search(query)

    @asyncme.run_or_none
    def async_search(self, query):
        """Asyncronous searching in PyPI"""
        for resource in self.get_selected_source().file_search(query):
            self.add_search_result(resource)
        self.search_finished()

    @asyncme.mainloop_only
    def add_search_result(self, resource):
        """Adding things to Gtk must be done in mainloop"""
        if isinstance(resource, RemotePage):
            return self.set_next_page(resource)

        self.results.add_item(resource)

    def search_clear(self, widget=None):
        """Remove previous search"""
        self.results.clear()
        self.update_btn_import()
        self.next_page_item = None
        self.widget('btn-next-page').hide()

    def search_started(self):
        """Set widgets to stun"""
        self.widget('dl-search').set_sensitive(False)
        self.widget('dl-searching').start()
        self.widget('dl-searching').show()

    @asyncme.mainloop_only
    def search_finished(self):
        """After everything, finish the search"""
        self.widget('dl-search').set_sensitive(True)
        self.widget('dl-searching').hide()
        self.widget('dl-searching').stop()

    def set_next_page(self, item):
        self.next_page_item = item
        self.widget('btn-next-page').show()

    def show_next_page(self, widget=None):
        item = self.next_page_item
        if item:
            self.search_clear()
            self.search_started()
            self.async_next_page(item)

    @asyncme.run_or_none
    def async_next_page(self, item):
        for resource in item.get_next_page():
            self.add_search_result(resource)
        self.search_finished()


class App(GtkApp):
    """Load the inkscape extensions glade file and attach to window"""
    glade_dir = os.path.join(os.path.dirname(__file__))
    app_name = 'inkscape-import-web-image'
    windows = [ImporterWindow]

class ImportWebImage(inkex.EffectExtension):
    """Import an svg from the internet"""
    selected_filename = None

    def merge_defs(self, defs):
        """Add all the items in defs to the self.svg.defs"""
        target = self.svg.defs
        for child in defs:
            if isinstance(child, StyleElement):
                continue # Already appled in merge_stylesheets()
            target.append(child)

    def merge_stylesheets(self, svg):
        """Because we don't want conflicting style-sheets (classes, ids, etc) we scrub them"""
        elems = defaultdict(list)
        # 1. Find all styles, and all elements that apply to them
        for sheet in svg.getroot().stylesheets:
            for style in sheet:
                xpath = style.to_xpath()
                for elem in svg.xpath(xpath):
                    elems[elem].append(style)
                    # 1b. Clear possibly conflicting attributes
                    if '@id' in xpath:
                        elem.set_random_id()
                    if '@class' in xpath:
                        elem.set('class', None)
        # 2. Apply each style cascade sequentially
        for elem, styles in elems.items():
            output = Style()
            for style in styles:
                output += style
            elem.style = output + elem.style

    def import_svg(self, new_svg):
        """Import an svg file into the current document"""
        self.merge_stylesheets(new_svg)
        for child in new_svg.getroot():
            if isinstance(child, SvgDocumentElement):
                yield from self.import_svg(child)
            elif isinstance(child, StyleElement):
                continue # Already applied in merge_stylesheets()
            elif isinstance(child, Defs):
                self.merge_defs(child)
            elif isinstance(child, (NamedView, Metadata)):
                pass
            else:
                yield child

    def import_from_file(self, filename):
        if not filename or not os.path.isfile(filename):
            return
        with open(filename, 'rb') as fhl:
            head = fhl.read(100)
            if b'<?xml' in head or b'<svg' in head:
                new_svg = load_svg(head + fhl.read())
                # Add each object to the container
                objs = list(self.import_svg(new_svg))

                if len(objs) == 1 and isinstance(objs[0], inkex.Group):
                    # Prevent too many groups, if item aready has one.
                    container = objs[0]
                else:
                    # Make a new group to contain everything
                    container = inkex.Group()
                    for child in objs:
                        container.append(child)

                # Retain the original filename as a group label
                container.label = os.path.basename(filename)
                # Apply unit transformation to keep things the same sizes.
                container.transform.add_scale(self.svg.unittouu(1.0) \
                    / new_svg.getroot().unittouu(1.0))


            else:
                container = self.import_raster(filename, fhl)

            self.svg.get_current_layer().append(container)

            # Make sure that none of the new content is a layer.
            for child in container.descendants():
                if isinstance(child, inkex.Group):
                    child.set("inkscape:groupmode", None)

    def effect(self):
        def select_func(filename):
            self.import_from_file(filename)

        App(start_loop=True, select=select_func)

    def import_raster(self, filename, handle):
        """Import a raster image"""
        # Don't read the whole file to check the header
        handle.seek(0)
        file_type = self.get_type(filename, handle.read(10))
        handle.seek(0)

        if file_type:
            # Future: Change encodestring to encodebytes when python3 only
            node = Image()
            node.label = os.path.basename(filename)
            node.set('xlink:href', 'data:{};base64,{}'.format(
                file_type, encodebytes(handle.read()).decode('ascii')))
            return node

    @staticmethod
    def get_type(path, header):
        """Basic magic header checker, returns mime type"""
        # Taken from embedimage.py
        for head, mime in (
                (b'\x89PNG', 'image/png'),
                (b'\xff\xd8', 'image/jpeg'),
                (b'BM', 'image/bmp'),
                (b'GIF87a', 'image/gif'),
                (b'GIF89a', 'image/gif'),
                (b'MM\x00\x2a', 'image/tiff'),
                (b'II\x2a\x00', 'image/tiff'),
            ):
            if header.startswith(head):
                return mime
        return None


if __name__ == '__main__':
    ImportWebImage().run()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Maren Hachmann <moini>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=arguments-differ
"""
Provide extra utility to each svg element type specific to its type.

This is useful for having a common interface for each element which can
give path, transform, and property access easily.
"""

from __future__ import annotations
import math

from typing import Optional

from lxml import etree

from ..styles import StyleSheet
from ..transforms import Vector2d, VectorLike, DirectedLineSegment

from ._base import BaseElement


class Defs(BaseElement):
    """A header defs element, one per document"""

    tag_name = "defs"


class StyleElement(BaseElement):
    """A CSS style element containing multiple style definitions"""

    tag_name = "style"

    def set_text(self, content):
        """Sets the style content text as a CDATA section"""
        self.text = etree.CDATA(str(content))

    def stylesheet(self):
        """Return the StyleSheet() object for the style tag"""
        return StyleSheet(self.text, callback=self.set_text)


class Script(BaseElement):
    """A javascript tag in SVG"""

    tag_name = "script"

    def set_text(self, content):
        """Sets the style content text as a CDATA section"""
        self.text = etree.CDATA(str(content))


class Desc(BaseElement):
    """Description element"""

    tag_name = "desc"


class Title(BaseElement):
    """Title element"""

    tag_name = "title"


class NamedView(BaseElement):
    """The NamedView element is Inkscape specific metadata about the file"""

    tag_name = "sodipodi:namedview"

    current_layer = property(lambda self: self.get("inkscape:current-layer"))

    @property
    def center(self):
        """Returns view_center in terms of document units"""
        return Vector2d(
            self.root.viewport_to_unit(self.get("inkscape:cx") or 0),
            self.root.viewport_to_unit(self.get("inkscape:cy") or 0),
        )

    def get_guides(self):
        """Returns a list of guides"""
        return self.findall("sodipodi:guide")

    def new_guide(self, position, orient=True, name=None):
        """Creates a new guide in this namedview

        Args:
            position: a float containing the y position for ``orient is True``, or
                the x position for ``orient is False``

                .. versionchanged:: 1.2
                    Alternatively, the position may be given as Tuple (or VectorLike)
            orient: True for horizontal, False for Vertical

                .. versionchanged:: 1.2
                    Tuple / Vector specifying x and y coordinates of the normal vector
                    of the guide.
            name: label of the guide

        Returns:
            the created guide"""
        if orient is True:
            elem = Guide().move_to(0, position, (0, 1))
        elif orient is False:
            elem = Guide().move_to(position, 0, (1, 0))
        else:
            elem = Guide().move_to(*position, orient)
        if name:
            elem.set("inkscape:label", str(name))
        return self.add(elem)

    def new_unique_guide(
        self, position: VectorLike, orientation: VectorLike
    ) -> Optional[Guide]:
        """Add a guide iif there is no guide that looks the same.

        .. versionadded:: 1.2"""
        elem = Guide().move_to(position[0], position[1], orientation)
        return self.add(elem) if self.get_similar_guide(elem) is None else None

    def get_similar_guide(self, other: Guide) -> Optional[Guide]:
        """Check if the namedview contains a guide that looks identical to one
        defined by (position, orientation). If such a guide exists, return it;
        otherwise, return None.

        .. versionadded:: 1.2"""
        for guide in self.get_guides():
            if Guide.guides_coincident(guide, other):
                return guide
        return None

    def get_pages(self):
        """Returns a list of pages

        .. versionadded:: 1.2"""
        return self.findall("inkscape:page")

    def new_page(self, x, y, width, height, label=None):
        """Creates a new page in this namedview

        .. versionadded:: 1.2"""
        elem = Page(width=width, height=height, x=x, y=y)
        if label:
            elem.set("inkscape:label", str(label))
        return self.add(elem)


class Guide(BaseElement):
    """An inkscape guide"""

    tag_name = "sodipodi:guide"

    @property
    def orientation(self) -> Vector2d:
        """Vector normal to the guide

        .. versionadded:: 1.2"""
        return Vector2d(self.get("orientation"), fallback=(1, 0))

    is_horizontal = property(
        lambda self: self.orientation[0] == 0 and self.orientation[1] != 0
    )
    is_vertical = property(
        lambda self: self.orientation[0] != 0 and self.orientation[1] == 0
    )

    @property
    def point(self) -> Vector2d:
        """Position of the guide handle. The y coordinate is flipped and relative
        to the bottom of the viewbox, this is a remnant of the pre-1.0 coordinate system
        """
        return Vector2d(self.get("position"), fallback=(0, 0))

    @classmethod
    def new(cls, pos_x, pos_y, angle, **attrs):
        guide = super().new(**attrs)
        guide.move_to(pos_x, pos_y, angle=angle)
        return guide

    def move_to(self, pos_x, pos_y, angle=None):
        """
        Move this guide to the given x,y position,

        Angle may be a float or integer, which will change the orientation. Alternately,
        it may be a pair of numbers (tuple) which will set the orientation directly.
        If not given at all, the orientation remains unchanged.
        """
        self.set("position", f"{float(pos_x):g},{float(pos_y):g}")
        if isinstance(angle, str):
            if "," not in angle:
                angle = float(angle)

        if isinstance(angle, (float, int)):
            # Generate orientation from angle
            angle = (math.sin(math.radians(angle)), -math.cos(math.radians(angle)))

        if isinstance(angle, (tuple, list)) and len(angle) == 2:
            angle = ",".join(f"{i:g}" for i in angle)

        if angle is not None:
            self.set("orientation", angle)
        return self

    @staticmethod
    def guides_coincident(guide1, guide2):
        """Check if two guides defined by (position, orientation) and (opos, oor) look
        identical (i.e. the position lies on the other guide AND the guide is
        (anti)parallel to the other guide).

        .. versionadded:: 1.2"""
        # normalize orientations first
        orientation = guide1.orientation / guide1.orientation.length
        oor = guide2.orientation / guide2.orientation.length

        position = guide1.point
        opos = guide2.point

        return (
            DirectedLineSegment(
                position, position + Vector2d(orientation[1], -orientation[0])
            ).perp_distance(*opos)
            < 1e-6
            and abs(abs(orientation[1] * oor[0]) - abs(orientation[0] * oor[1])) < 1e-6
        )


class Metadata(BaseElement):
    """Inkscape Metadata element"""

    tag_name = "metadata"


class ForeignObject(BaseElement):
    """SVG foreignObject element"""

    tag_name = "foreignObject"


class Switch(BaseElement):
    """A switch element"""

    tag_name = "switch"


class Grid(BaseElement):
    """A namedview grid child"""

    tag_name = "inkscape:grid"


class Page(BaseElement):
    """A namedview page child

    .. versionadded:: 1.2"""

    tag_name = "inkscape:page"

    width = property(lambda self: self.to_dimensionless(self.get("width") or 0))
    height = property(lambda self: self.to_dimensionless(self.get("height") or 0))
    x = property(lambda self: self.to_dimensionless(self.get("x") or 0))
    y = property(lambda self: self.to_dimensionless(self.get("y") or 0))

    @classmethod
    def new(cls, width, height, x, y):
        """Creates a new page element in the namedview"""
        page = super().new()
        page.move_to(x, y)
        page.set("width", width)
        page.set("height", height)
        return page

    def move_to(self, x, y):
        """Move this page to the given x,y position"""
        self.set("position", f"{float(x):g},{float(y):g}")
        return self



# coding=utf-8
#
# Copyright (C) 2018 - Martin Owens <doctormo@mgail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Deprecation functionality for the pre-1.0 Inkex main effect class.
"""
#
# We ignore a lot of pylint warnings here:
#
# pylint: disable=invalid-name,unused-argument,missing-docstring,too-many-public-methods
#

import sys
import argparse
from argparse import ArgumentParser

from .. import utils
from .. import base
from ..base import SvgThroughMixin, InkscapeExtension
from ..localization import inkex_gettext as _
from .meta import _deprecated


class DeprecatedEffect:
    """An Inkscape effect, takes SVG in and outputs SVG, providing a deprecated layer"""

    options = argparse.Namespace()

    def __init__(self):
        super().__init__()
        self._doc_ids = None
        self._args = None

        # These are things we reference in the deprecated code, they are provided
        # by the new effects code, but we want to keep this as a Mixin so these
        # items will keep pylint happy and let use check our code as we write.
        if not hasattr(self, "svg"):
            from ..elements import SvgDocumentElement

            self.svg = SvgDocumentElement()
        if not hasattr(self, "arg_parser"):
            self.arg_parser = ArgumentParser()
        if not hasattr(self, "run"):
            self.run = self.affect

    @classmethod
    def _deprecated(
        cls, name, msg=_("{} is deprecated and should be removed"), stack=3
    ):
        """Give the user a warning about their extension using a deprecated API"""
        _deprecated(
            msg.format("Effect." + name, cls=cls.__module__ + "." + cls.__name__),
            stack=stack,
        )

    @property
    def OptionParser(self):
        self._deprecated(
            "OptionParser",
            _(
                "{} or `optparse` has been deprecated and replaced with `argparser`. "
                "You must change `self.OptionParser.add_option` to "
                "`self.arg_parser.add_argument`; the arguments are similar."
            ),
        )
        return self

    def add_option(self, *args, **kw):
        # Convert type string into type method as needed
        if "type" in kw:
            kw["type"] = {
                "string": str,
                "int": int,
                "float": float,
                "inkbool": utils.Boolean,
            }.get(kw["type"])
        if kw.get("action", None) == "store":
            # Default store action not required, removed.
            kw.pop("action")
        args = [arg for arg in args if arg != ""]
        self.arg_parser.add_argument(*args, **kw)

    def effect(self):
        self._deprecated(
            "effect",
            _(
                "{} method is now a required method. It should "
                "be created on {cls}, even if it does nothing."
            ),
        )

    @property
    def current_layer(self):
        self._deprecated(
            "current_layer",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.get_current_layer()` instead."
            ),
        )
        return self.svg.get_current_layer()

    @property
    def view_center(self):
        self._deprecated(
            "view_center",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.get_center_position()` instead."
            ),
        )
        return self.svg.namedview.center

    @property
    def selected(self):
        self._deprecated(
            "selected",
            _(
                "{} is now a dict in the SvgDocumentElement class. "
                "Use `self.svg.selected`."
            ),
        )
        return {elem.get("id"): elem for elem in self.svg.selected}

    @property
    def doc_ids(self):
        self._deprecated(
            "doc_ids",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.get_ids()` instead."
            ),
        )
        if self._doc_ids is None:
            self._doc_ids = dict.fromkeys(self.svg.get_ids())
        return self._doc_ids

    def getdocids(self):
        self._deprecated(
            "getdocids", _("Use `self.svg.get_ids()` instead of {} and `doc_ids`.")
        )
        self._doc_ids = None
        self.svg.ids.clear()

    def getselected(self):
        self._deprecated("getselected", _("{} has been removed"))

    def getElementById(self, eid):
        self._deprecated(
            "getElementById",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.getElementById(eid)` instead."
            ),
        )
        return self.svg.getElementById(eid)

    def xpathSingle(self, xpath):
        self._deprecated(
            "xpathSingle",
            _(
                "{} is now a new method in the SvgDocumentElement class. "
                "Use `self.svg.getElement(path)` instead."
            ),
        )
        return self.svg.getElement(xpath)

    def getParentNode(self, node):
        self._deprecated(
            "getParentNode",
            _("{} is no longer in use. Use the lxml `.getparent()` method instead."),
        )
        return node.getparent()

    def getNamedView(self):
        self._deprecated(
            "getNamedView",
            _(
                "{} is now a property of the SvgDocumentElement class. "
                "Use `self.svg.namedview` to access this element."
            ),
        )
        return self.svg.namedview

    def createGuide(self, posX, posY, angle):
        from ..elements import Guide

        self._deprecated(
            "createGuide",
            _(
                "{} is now a method of the namedview element object. "
                "Use `self.svg.namedview.add(Guide().move_to(x, y, a))` instead."
            ),
        )
        return self.svg.namedview.add(Guide().move_to(posX, posY, angle))

    def affect(
        self, args=sys.argv[1:], output=True
    ):  # pylint: disable=dangerous-default-value
        # We need a list as the default value to preserve backwards compatibility
        self._deprecated(
            "affect", _("{} is now `Effect.run()`. The `output` argument has changed.")
        )
        self._args = args[-1:]
        return self.run(args=args)

    @property
    def args(self):
        self._deprecated("args", _("self.args[-1] is now self.options.input_file."))
        return self._args

    @property
    def svg_file(self):
        self._deprecated("svg_file", _("self.svg_file is now self.options.input_file."))
        return self.options.input_file

    def save_raw(self, ret):
        # Derived class may implement "output()"
        # Attention: 'cubify.py' implements __getattr__ -> hasattr(self, 'output')
        # returns True
        if hasattr(self.__class__, "output"):
            self._deprecated("output", "Use `save()` or `save_raw()` instead.", stack=5)
            return getattr(self, "output")()
        return base.InkscapeExtension.save_raw(self, ret)

    def uniqueId(self, old_id, make_new_id=True):
        self._deprecated(
            "uniqueId",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                " Use `self.svg.get_unique_id(old_id)` instead."
            ),
        )
        return self.svg.get_unique_id(old_id)

    def getDocumentWidth(self):
        self._deprecated(
            "getDocumentWidth",
            _(
                "{} is now a property of the SvgDocumentElement class. "
                "Use `self.svg.width` instead."
            ),
        )
        return self.svg.get("width")

    def getDocumentHeight(self):
        self._deprecated(
            "getDocumentHeight",
            _(
                "{} is now a property of the SvgDocumentElement class. "
                "Use `self.svg.height` instead."
            ),
        )
        return self.svg.get("height")

    def getDocumentUnit(self):
        self._deprecated(
            "getDocumentUnit",
            _(
                "{} is now a property of the SvgDocumentElement class. "
                "Use `self.svg.unit` instead."
            ),
        )
        return self.svg.unit

    def unittouu(self, string):
        self._deprecated(
            "unittouu",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.unittouu(str)` instead."
            ),
        )
        return self.svg.unittouu(string)

    def uutounit(self, val, unit):
        self._deprecated(
            "uutounit",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.uutounit(value, unit)` instead."
            ),
        )
        return self.svg.uutounit(val, unit)

    def addDocumentUnit(self, value):
        self._deprecated(
            "addDocumentUnit",
            _(
                "{} is now a method in the SvgDocumentElement class. "
                "Use `self.svg.add_unit(value)` instead."
            ),
        )
        return self.svg.add_unit(value)


class Effect(SvgThroughMixin, DeprecatedEffect, InkscapeExtension):
    """An Inkscape effect, takes SVG in and outputs SVG"""



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Thomas Holder <thomas.holder@schrodinger.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=arguments-differ
"""
Provide text based element classes interface.

Because text is not rendered at all, no information about a text's path
size or actual location can be generated yet.
"""
from __future__ import annotations

from tempfile import TemporaryDirectory

from ..interfaces.IElement import BaseElementProtocol
from ..paths import Path
from ..transforms import Transform, BoundingBox
from ..command import inkscape, write_svg
from ._base import BaseElement, ShapeElement
from ._polygons import PathElementBase


class TextBBMixin:  # pylint: disable=too-few-public-methods
    """Mixin to query the bounding box from Inkscape

    .. versionadded:: 1.2"""

    def get_inkscape_bbox(self: BaseElementProtocol) -> BoundingBox:
        """Query the bbbox of a single object. This calls the Inkscape command,
        so it is rather slow to use in a loop."""
        with TemporaryDirectory(prefix="inkscape-command") as tmpdir:
            svg_file = write_svg(self.root, tmpdir, "input.svg")
            out = inkscape(svg_file, "-X", "-Y", "-W", "-H", query_id=self.get_id())
            out = list(map(self.root.viewport_to_unit, out.splitlines()))
            if len(out) != 4:
                raise ValueError("Error: Bounding box computation failed")
            return BoundingBox.new_xywh(*out)


class FlowRegion(ShapeElement):
    """SVG Flow Region (SVG 2.0)"""

    tag_name = "flowRegion"

    def get_path(self):
        # This ignores flowRegionExcludes
        return sum([child.path for child in self], Path())


class FlowRoot(ShapeElement, TextBBMixin):
    """SVG Flow Root (SVG 2.0)"""

    tag_name = "flowRoot"

    @property
    def region(self):
        """Return the first flowRegion in this flowRoot"""
        return self.findone("svg:flowRegion")

    def get_path(self):
        region = self.region
        return region.get_path() if region is not None else Path()


class FlowPara(ShapeElement):
    """SVG Flow Paragraph (SVG 2.0)"""

    tag_name = "flowPara"

    def get_path(self):
        # XXX: These empty paths mean the bbox for text elements will be nothing.
        return Path()


class FlowDiv(ShapeElement):
    """SVG Flow Div (SVG 2.0)"""

    tag_name = "flowDiv"

    def get_path(self):
        # XXX: These empty paths mean the bbox for text elements will be nothing.
        return Path()


class FlowSpan(ShapeElement):
    """SVG Flow Span (SVG 2.0)"""

    tag_name = "flowSpan"

    def get_path(self):
        # XXX: These empty paths mean the bbox for text elements will be nothing.
        return Path()


class TextElement(ShapeElement, TextBBMixin):
    """A Text element"""

    tag_name = "text"
    x = property(lambda self: self.to_dimensionless(self.get("x", 0)))
    y = property(lambda self: self.to_dimensionless(self.get("y", 0)))

    def get_path(self):
        return Path()

    def tspans(self):
        """Returns all children that are tspan elements"""
        return self.findall("svg:tspan")

    def get_text(self, sep="\n"):
        """Return the text content including tspans"""
        nodes = [self] + list(self.tspans())
        return sep.join([elem.text for elem in nodes if elem.text is not None])

    def shape_box(self, transform=None):
        """
        Returns a horrible bounding box that just contains the coord points
        of the text without width or height (which is impossible to calculate)
        """
        effective_transform = Transform(transform) @ self.transform
        x, y = effective_transform.apply_to_point((self.x, self.y))
        bbox = BoundingBox(x, y)
        for tspan in self.tspans():
            bbox += tspan.bounding_box(effective_transform)
        return bbox


class TextPath(ShapeElement, TextBBMixin):
    """A textPath element"""

    tag_name = "textPath"

    def get_path(self):
        return Path()


class Tspan(ShapeElement, TextBBMixin):
    """A tspan text element"""

    tag_name = "tspan"
    x = property(lambda self: self.to_dimensionless(self.get("x", 0)))
    y = property(lambda self: self.to_dimensionless(self.get("y", 0)))

    @classmethod
    def superscript(cls, text):
        """Adds a superscript tspan element"""
        return cls(text, style="font-size:65%;baseline-shift:super")

    def get_path(self):
        return Path()

    def shape_box(self, transform=None):
        """
        Returns a horrible bounding box that just contains the coord points
        of the text without width or height (which is impossible to calculate)
        """
        effective_transform = Transform(transform) @ self.transform
        x1, y1 = effective_transform.apply_to_point((self.x, self.y))
        fontsize = self.to_dimensionless(self.style.get("font-size", "12px"))
        x2 = self.x + 0  # XXX This is impossible to calculate!
        y2 = self.y + float(fontsize)
        x2, y2 = effective_transform.apply_to_point((x2, y2))
        return BoundingBox((x1, x2), (y1, y2))


class SVGfont(BaseElement):
    """An svg font element"""

    tag_name = "font"


class FontFace(BaseElement):
    """An svg font font-face element"""

    tag_name = "font-face"


class Glyph(PathElementBase):
    """An svg font glyph element"""

    tag_name = "glyph"


class MissingGlyph(BaseElement):
    """An svg font missing-glyph element"""

    tag_name = "missing-glyph"



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Extension to sssign jessyInk views to objects"""

import inkex

from jessyink_install import JessyInkMixin, _


class View(JessyInkMixin, inkex.EffectExtension):
    """Assign jessyInk views to objects"""

    def add_arguments(self, pars):
        pars.add_argument("--tab", dest="what")
        pars.add_argument("--viewOrder", type=int, default=1)
        pars.add_argument("--viewDuration", type=float, default=0.8)
        pars.add_argument("--removeView", type=inkex.Boolean)

    def effect(self):
        self.is_installed()

        rect = self.svg.selection.first()

        if rect is None:
            raise inkex.AbortExtension(
                _(
                    "No object selected. Please select the object you want "
                    "to assign a view to and then press apply.\n"
                )
            )

        if not self.options.removeView:
            view_order = str(self.options.viewOrder)
            # Remove the view that currently has the requested order number.
            for node in rect.xpath(
                "ancestor::svg:g[@inkscape:groupmode='layer']"
                "/descendant::*[@jessyink:view]"
            ):
                prop_dict = inkex.Style(node.get("jessyink:view"))

                if prop_dict["order"] == view_order:
                    node.set("jessyink:view", None)

            # Set the new view.
            rect.set(
                "jessyink:view",
                inkex.Style(
                    name="view",
                    order=view_order,
                    length=int(self.options.viewDuration * 1000),
                ),
            )

            # Remove possible effect arguments.
            self.attr_remove("effectIn")
            self.attr_remove("effectOut")
        else:
            self.attr_remove("view")


if __name__ == "__main__":
    View().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007-2019 Matt Harrison, matthewharrison [at] gmail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
A script that slices images.  It might be useful for web design.

You pass it the name of a layer containing rectangles that cover
the areas that you want exported (the default name for this layer
is "slices").  It then sets the opacity to 0 for all the rectangles
defined in that layer and exports as png whatever they covered.
The output filenames are based on the "Id" field of "Object Properties"
right click contextual menu of the rectangles.

One side effect is that after exporting, it sets the slice rectangles
to different colors with a 25% opacity.  (If you want to hide them,
just click on the eye next to the layer).

  * red - overwrote a file
  * green - wrote a new file
  * grey - skipped (not overwriting)

For good pixel exports set the Document Properties, default units to "px"
and the width/height to the real size. (I use 1024x768)

Here's the process I've used for slicing web layout with
Inkscape: Create your webpage layout (set page units to "px",
width/height appropriately and snap to 1 pixel intervals. This should
allow pixel perfect alignment). Then create a new layer, naming it
slices. Draw rectangles over the areas you want to slice (set
x,y,width,height to whole pixel values). Name these rectangles using
the Object Properties found in the right click contextual menu (the
saved images name will be based on that value, so name them something
like "header" instead of the default/non-useful "rect4312").
"""
import os
import tempfile

import inkex
from inkex.command import inkscape
from inkex.localization import inkex_gettext as _


class ExportSlices(inkex.EffectExtension):
    """Exports all rectangles in the current layer"""

    GREEN = "#00ff00"  # new export
    GREY = "#555555"  # not exported
    RED = "#ff0000"  # overwrite

    def __init__(self):
        super(ExportSlices, self).__init__()
        self.color_map = {}  # map node id to color based on overwrite

    def add_arguments(self, pars):
        pars.add_argument("--tab")
        pars.add_argument(
            "--directory",
            default=os.path.expanduser("~"),
            help="Existing destination directory",
        )
        pars.add_argument(
            "--layer", default="slices", help="Layer with slices (rects) in it"
        )
        pars.add_argument("--iconmode", type=inkex.Boolean, help="Icon export mode")
        pars.add_argument(
            "--sizes",
            default="128, 64, 48, 32, 24, 16",
            help="sizes to export comma separated",
        )
        pars.add_argument(
            "--overwrite", type=inkex.Boolean, help="Overwrite existing exports?"
        )
        pars.add_argument("--dpi", default="300", help="Dots per inch (300 default)")

    def effect(self):
        if not os.path.isdir(self.options.directory):
            os.makedirs(self.options.directory)

        nodes = self.get_layer_nodes(self.options.layer)
        if nodes is None:
            raise inkex.AbortExtension(
                _("Slice: '{}' does not exist.").format(self.options.layer)
            )

        # set opacity to zero in slices
        for node in nodes:
            self.clear_color(node)

        # save file once now
        # if we have multiple slices we will make multiple calls
        # to inkscape
        (__, tmp_svg) = tempfile.mkstemp(".svg")
        with open(tmp_svg, "wb") as fout:
            fout.write(self.svg.tostring())

        # in case there are overlapping rects, clear them all out before
        # saving any
        for node in nodes:
            if self.options.iconmode:
                for size in self.options.sizes.split(","):
                    size = size.strip()
                    if size.isdigit():
                        png_size = int(size)
                        self.export_node(node, png_size, png_size)
            else:
                self.export_node(node)

        # change slice colors to grey/green/red and set opacity to 25% in real document
        for node in nodes:
            self.change_color(node)
        return self.document

    def get_layer_nodes(self, layer_name):
        """
        given the name of a layer one that contains the rectangles defining slices,
        return the nodes of the rectangles.
        """
        # get layer we intend to slice
        slice_node = None
        slice_layer = self.svg.findall("svg:g")
        for node in slice_layer:
            label_value = node.label
            if label_value == layer_name:
                slice_node = node

        if slice_node is not None:
            return slice_node.findall("svg:rect")
        return slice_node

    def clear_color(self, node):
        """
        set opacity to zero, and stroke to none

        Node looks like this:
        <rect
        style="opacity:0;fill:#eeeeec;fill-opacity:1;stroke:none;stroke-width:4.00099993;stroke-linecap:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1;display:inline"
        """
        node.style.update({"stroke": "none", "opacity": "0"})

    def change_color(self, node):
        """
        set color from color_map and set opacity to 25%

        """
        node_id = node.attrib["id"]
        color = self.color_map[node_id]
        node.style.update({"fill": color, "opacity": ".25"})

    def export_node(self, node, height=None, width=None):
        color, kwargs = self.get_color_and_command_kwargs(node, height, width)
        node_id = node.attrib["id"]
        self.color_map[node_id] = color
        if color == ExportSlices.GREY:  # skipping
            return
        svg_file = self.options.input_file
        inkscape(svg_file, **kwargs)

    def get_color_and_command_kwargs(self, node, height=None, width=None):
        directory = self.options.directory
        node_id = node.attrib["id"]
        size = "" if height is None else "-{}x{}".format(width, height)
        file_name = "{}{}.png".format(node_id, size)
        filename = os.path.join(directory, file_name)
        color = ExportSlices.GREY  # skipping
        if self.options.overwrite or not os.path.exists(filename):
            color = ExportSlices.RED  #  overwritten
            if not os.path.exists(filename):
                color = ExportSlices.GREEN  # new export
            kwargs = {
                "export-id": node_id,
                "export-filename": filename,
                "export-dpi": self.options.dpi,
            }
            if width:
                kwargs["export-height"] = str(height)
                kwargs["export-width"] = str(width)
            return color, kwargs
        else:
            inkex.errormsg(_("Export exists ({}), not overwriting").format(filename))
            return color, {}


if __name__ == "__main__":
    ExportSlices().run()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Sergei Izmailov <sergei.a.izmailov@gmail.com>
#                    Thomas Holder <thomas.holder@schrodinger.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

"""Utilities for parsing SVG documents.

.. versionadded:: 1.2
    Separated out from :py:mod:`inkex.elements._base`"""

from collections import defaultdict
from typing import DefaultDict, List, Any, Type

from lxml import etree

from ..interfaces.IElement import IBaseElement

from ._utils import splitNS


class NodeBasedLookup(etree.PythonElementClassLookup):
    """
    We choose what kind of Elements we should return for each element, providing useful
    SVG based API to our extensions system.
    """

    default: Type[IBaseElement]

    # (ns,tag) -> list(cls) ; ascending priority
    lookup_table = defaultdict(list)  # type: DefaultDict[str, List[Any]]

    @classmethod
    def register_class(cls, klass):
        """Register the given class using it's attached tag name"""
        cls.lookup_table[splitNS(klass.tag_name)].append(klass)

    @classmethod
    def find_class(cls, xpath):
        """Find the class for this type of element defined by an xpath

        .. versionadded:: 1.1"""
        if isinstance(xpath, type):
            return xpath
        for kls in cls.lookup_table[splitNS(xpath.split("/")[-1])]:
            # TODO: We could create a apply the xpath attrs to the test element
            # to narrow the search, but this does everything we need right now.
            test_element = kls()
            if kls.is_class_element(test_element):
                return kls
        raise KeyError(f"Could not find svg tag for '{xpath}'")

    def lookup(self, doc, element):  # pylint: disable=unused-argument
        """Lookup called by lxml when assigning elements their object class"""
        try:
            for kls in reversed(self.lookup_table[splitNS(element.tag)]):
                if kls.is_class_element(element):  # pylint: disable=protected-access
                    return kls
        except TypeError:
            # Handle non-element proxies case
            # The documentation implies that it's not possible
            # Didn't found a reliable way to check whether proxy corresponds to element
            # or not
            # Look like lxml issue to me.
            # The troubling element is "<!--Comment-->"
            return None
        return NodeBasedLookup.default


SVG_PARSER = etree.XMLParser(huge_tree=True, strip_cdata=False)
SVG_PARSER.set_element_class_lookup(NodeBasedLookup())


def load_svg(stream):
    """Load SVG file using the SVG_PARSER"""
    if (isinstance(stream, str) and stream.lstrip().startswith("<")) or (
        isinstance(stream, bytes) and stream.lstrip().startswith(b"<")
    ):
        return etree.ElementTree(etree.fromstring(stream, parser=SVG_PARSER))
    return etree.parse(stream, parser=SVG_PARSER)



#
# Copyright (C) 2019 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Support for constructing and understanding installed extensions
as python packages.

Start by using 'PackageLists' and iterate over to get 'PackageList' objects
iterate over those to get 'Package' objects and use those to know more
information about the installed packages.
"""

import os
import sys
import json
import logging
from fnmatch import fnmatch

from .utils import (
    DATA_DIR,
    ICON_SEP,
    parse_metadata,
    clean_author,
    get_user_directory,
    get_inkscape_directory,
    ExtensionInx,
)

# The ORDER of these places is important, when installing packages it will
# choose the FIRST writable directory in the lsit, so make sure that the
# more preferable locations appear at the top.
EXTENSION_PLACES = []

for path in sys.path:
    if "extensions" in path:
        if "/bin" in path:
            path = path.replace("/bin", "")
        EXTENSION_PLACES.append(path)

CORE_ICON = os.path.join(DATA_DIR, "pixmaps", "core_icon.svg")
ORPHAN_ICON = os.path.join(DATA_DIR, "pixmaps", "orphan_icon.svg")
DEFAULT_ICON = os.path.join(DATA_DIR, "pixmaps", "default_icon.svg")
MODULE_ICON = os.path.join(DATA_DIR, "pixmaps", "module_icon.svg")


class PackageItem(object):
    """
    Gets information about the package using requests.

    Can be remote, or local json file.
    """

    default_icon = DEFAULT_ICON

    @classmethod
    def is_pkg(cls, info):
        """Returns true if this info package is considered a managed package"""
        for field in ("name", "author", "verified", "links", "summary"):
            if field not in info:
                return False
        return True

    def __init__(self, info, remote=None):
        if not self.is_pkg(info):
            raise ValueError("Not a valid package, refusing to create it.")

        self.info = info
        self.remote = remote
        self.installed = None
        self._installer = None
        self._uninstaller = None
        self._missing = []
        self._icon = None

    ident = property(lambda self: self.info.get("id"))
    url = property(lambda self: self.info["links"]["file"])
    link = property(lambda self: self.info["links"]["html"])
    name = property(lambda self: self.info["name"] or "Unnamed")
    license = property(lambda self: self.info.get("license") or "")
    author = property(lambda self: self.info["author"])
    tags = property(lambda self: self.info.get("tags", []))

    stars = property(lambda self: self.info["stats"]["liked"])
    downloads = property(lambda self: self.info["stats"]["downloaded"])
    verified = property(lambda self: self.info["verified"])
    recommended = property(lambda self: self.info["stats"].get("extra", 0) == 7)

    targets = property(lambda self: self.info.get("Inkscape Version", []))
    target = property(lambda self: ", ".join(self.targets))

    @property
    def summary(self):
        ret = self.info["summary"] or "No summary"
        if len(ret) > 110:
            ret = ret.split(". ", 1)[0]
        if len(ret) > 110:
            ret = ret[:110].rsplit(" ", 1)[0] + "..."
        return ret

    @property
    def version(self):
        if "version" in self.info:
            return str(self.info["version"])
        if "stats" in self.info and "revisions" in self.info["stats"]:
            return str(int(self.info["stats"]["revisions"]) + 1)
        return "?"

    def set_installer(self, fn):
        self._installer = fn

    def set_uninstaller(self, fn, *args):
        def _inner(info):
            return fn(info, *args)

        self._uninstaller = _inner

    def set_installed(self, installed=True):
        self.installed = installed

    def is_installable(self, version):
        if version not in self.targets or not self.verified:
            return False
        return bool(not self.installed and self._installer)

    def is_uninstallable(self):
        return bool(self._uninstaller)

    def install(self):
        """Install the remote package"""
        if not self.remote:
            raise IOError("Can not install without a defined remote")
        url = self.remote(self.info["links"]["file"])
        msg = self._installer(url.as_local_file(), self.info)
        self.set_installed(True)
        return msg

    def uninstall(self):
        """Remove the pakage if possible"""
        if self._uninstaller(self.info):
            self.set_installed(False)
            return True
        return False

    def get_icon(self):
        if self._icon:
            return self._icon
        if self.info.get("icon"):
            if not self.remote:
                raise IOError("Can get icon without a defined remote")
            icon = self.remote(self.info["icon"])
            self._icon = icon.as_local_file() or self.default_icon
            return self._icon
        for filename in self.get_files():
            name = os.path.basename(filename)
            if name in ("icon.svg", "icon.png"):
                self._icon = os.path.abspath(filename)
                return filename
        return self.default_icon

    def get_files(self, missing=False):
        """List files"""
        return [
            name
            for name in self.info.get("files", [])
            if missing or name not in self._missing
        ]


class OrphanedItem(PackageItem):
    """
    A special kind of item to collect all orphaned files
    """

    default_icon = ORPHAN_ICON

    def __init__(self, parent):
        super().__init__(
            {
                "name": "Orphan Extensions",
                "author": "Various",
                "verified": False,
                "summary": "Old and manually installed extensions",
                "license": "Unknown",
                "links": {},
                "version": "-",
                "stats": {"liked": -1, "downloaded": -1},
            }
        )

        self._icon = self.default_icon
        self._parent = os.path.abspath(parent)
        self._files = set()
        self._removed = set()
        self._items = {}

    def _get_target_file(self, filename):
        if not os.path.isabs(filename):
            filename = os.path.join(self._parent, filename)
        path = os.path.abspath(filename)
        return path.replace(self._parent, "").lstrip("/")

    def add_file(self, filename):
        """Add a file to the 'this file exists' list"""
        self._files.add(self._get_target_file(filename))

    def remove_file(self, filename, item=None):
        """Add a file to the 'this file is said to exist' list"""
        fn = self._get_target_file(filename)
        self._removed.add(fn)
        if item is not None:
            self._items[fn] = item

    def get_files(self, filters=()):
        """Returns a filtered set of files which exist, minus ones said to exist"""
        if not filters:
            filters = ("*",)
        items = []
        for item in self._files - self._removed:
            if any([fnmatch(item, filt) for filt in filters]):
                items.append(item)
        return items

    def get_missing(self):
        """Returns a set of files which don't exist, even though they were said to"""
        return [(fn, self._items.get(fn, None)) for fn in self._removed - self._files]


class PythonItem(PackageItem):
    """
    A python package that has an inx file, but was never installed properly.
    """

    def __init__(self, pip):
        self.pip = pip
        info = self.pip.get_metadata()
        super().__init__(
            {
                "pip": True,
                "name": info["name"],
                "summary": info["summary"],
                "author": info["author"] or "Unknown",
                "version": info["version"],
                "license": info["license"],
                "verified": False,
                "links": {},
                "stats": {"liked": -1, "downloaded": -1},
            }
        )

    def get_files(self):
        return self.pip.package_files()


class PythonPackage(object):
    """
    A reprisentation of the python package, NOT the json based packages.
    """

    name = property(lambda self: self.get_metadata()["name"])
    version = property(lambda self: self.get_metadata()["version"])

    def __init__(self, path, parent):
        self._metadata = None
        self.path = path
        self.parent = parent

    def get_inx(self):
        """Yields all of the inx files in this package"""
        for filename in self.package_files():
            if filename.endswith(".inx"):
                yield filename

    def package_files(self):
        """Return a generator of all files in this installed package"""
        parent = os.path.abspath(os.path.dirname(self.path))
        record = self.get_file("RECORD")
        if not record:
            return
        for line in record.split("\n"):
            if line and "," in line:
                (filename, checksum, size) = line.rsplit(",", 2)
                # XXX Check filesize or checksum?
                if not os.path.isabs(filename):
                    filename = os.path.join(parent, filename)
                yield filename

    def get_metadata(self):
        """Returns the metadata from an array of known types"""
        if self._metadata is None:
            for name in ("METADATA", "PKG-INFO"):
                md_mail = self.get_file(name)
                if md_mail:
                    self._metadata = parse_metadata(md_mail)

        if self._metadata is None:
            md_json = self.get_file("metadata.json")
            if md_json:
                self._metadata = clean_author(json.loads(md_json))

        if self._metadata is None:
            raise KeyError("Can't find package meta data: {}".format(self.path))
        return self._metadata

    def get_file(self, name):
        """Get filename if it exists"""
        if not self.path or not os.path.isdir(self.path):
            return None
        path = os.path.join(self.path, name)
        if os.path.isfile(path):
            with open(path, "r") as fhl:
                return fhl.read()
        return None

    def get_depedencies(self):
        """Return a list of other pip packages this packages needs"""
        return self.get_metadata()["requires"]



#!/usr/bin/env python
"""Make text lower case"""

import inkex


class Lowercase(inkex.TextExtension):
    """Convert to lowercase"""

    def process_chardata(self, text):
        return text.lower()


if __name__ == "__main__":
    Lowercase().run()



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#                 2020 Martin Owens
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Jessyink effect extension."""

import inkex

from jessyink_install import JessyInkMixin, _


class JessyinkEffects(JessyInkMixin, inkex.EffectExtension):
    """Add ad effect to jessy ink selected items"""

    def add_arguments(self, pars):
        pars.add_argument("--tab")
        pars.add_argument("--effectInOrder", type=int, default=1)
        pars.add_argument("--effectInDuration", type=float, default=0.8)
        pars.add_argument("--effectIn", default="none")
        pars.add_argument("--effectOutOrder", type=int, default=1)
        pars.add_argument("--effectOutDuration", type=float, default=0.8)
        pars.add_argument("--effectOut", default="none")

    def effect(self):
        self.is_installed()
        if not self.svg.selection:
            raise inkex.AbortExtension(
                _(
                    "No object selected. Please select the object you want to "
                    "assign an effect to and then press apply.\n"
                )
            )

        for elem in self.svg.selection.values():
            self._process(elem, "effectIn")
            self._process(elem, "effectOut")

    def _process(self, elem, name):
        effect = getattr(self.options, name)
        order = getattr(self.options, name + "Order")
        duration = int(getattr(self.options, name + "Duration") * 1000)

        if effect in ("appear", "fade", "pop"):
            elem.set(
                "jessyink:" + name,
                inkex.Style(name=effect, order=order, length=duration),
            )
            # Remove possible view argument.
            elem.pop("jessyink:view", None)
        else:
            elem.pop("jessyink:" + name, None)


if __name__ == "__main__":
    JessyinkEffects().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2010 Jean-Luc JOULIN "JeanJouX" jean-luc.joulin@laposte.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
This extension allow you to draw a isometric grid with inkscape
There is some options including subdivision, subsubdivions and custom line width
All elements are grouped with similar elements
These grid are used for isometric view in mechanical drawing or piping schematic
!!! Y Divisions can't be smaller than half the X Divions
"""

import inkex
from inkex import Rectangle
from inkex.paths import Move, Line


def draw_line(x1, y1, x2, y2, width, name, parent):
    elem = parent.add(inkex.PathElement())
    elem.style = {"stroke": "#000000", "stroke-width": str(width), "fill": "none"}
    elem.set("inkscape:label", name)
    elem.path = [Move(x1, y1), Line(x2, y2)]


def draw_rect(x, y, w, h, width, fill, name):
    elem = Rectangle(x=str(x), y=str(y), width=str(w), height=str(h))
    elem.style = {"stroke": "#000000", "stroke-width": str(width), "fill": fill}
    elem.set("inkscape:label", name)
    return elem


class GridIsometric(inkex.GenerateExtension):
    def add_arguments(self, pars):
        pars.add_argument(
            "--x_divs", type=int, dest="x_divs", default=5, help="Major X Divisions"
        )
        pars.add_argument(
            "--y_divs", type=int, dest="y_divs", default=5, help="Major Y Divisions"
        )
        pars.add_argument(
            "--dx", type=float, dest="dx", default=50.0, help="Major X division Spacing"
        )
        pars.add_argument(
            "--subdivs",
            type=int,
            dest="subdivs",
            default=2,
            help="Subdivisions per Major X division",
        )
        pars.add_argument(
            "--subsubdivs",
            type=int,
            dest="subsubdivs",
            default=5,
            help="Subsubdivisions per Minor X division",
        )
        pars.add_argument(
            "--divs_th",
            type=float,
            dest="divs_th",
            default=2,
            help="Major X Division Line thickness",
        )
        pars.add_argument(
            "--subdivs_th",
            type=float,
            dest="subdivs_th",
            default=0.5,
            help="Minor X Division Line thickness",
        )
        pars.add_argument(
            "--subsubdivs_th",
            type=float,
            dest="subsubdivs_th",
            default=0.1,
            help="Subminor X Division Line thickness",
        )
        pars.add_argument(
            "--border_th",
            type=float,
            dest="border_th",
            default=3,
            help="Border Line thickness",
        )

    @property
    def container_label(self):
        """Generate label from options"""
        return "Grid_Polar:X{0.x_divs}:Y{0.y_divs}".format(
            self.options
        )  # pylint: disable=missing-format-attribute

    def generate(self):
        self.options.dx = self.svg.unittouu(str(self.options.dx) + "px")
        self.options.divs_th = self.svg.unittouu(str(self.options.divs_th) + "px")
        self.options.subdivs_th = self.svg.unittouu(str(self.options.subdivs_th) + "px")
        self.options.subsubdivs_th = self.svg.unittouu(
            str(self.options.subsubdivs_th) + "px"
        )
        self.options.border_th = self.svg.unittouu(str(self.options.border_th) + "px")

        # Can't generate a grid too flat
        # If the Y dimension is smallest than half the X dimension, fix it.
        if self.options.y_divs < ((self.options.x_divs + 1) / 2):
            self.options.y_divs = int((self.options.x_divs + 1) / 2)

        # Find the pixel dimensions of the overall grid
        xmax = self.options.dx * (2 * self.options.x_divs)
        ymax = self.options.dx * (2 * self.options.y_divs) / 0.866025

        # Group for major x gridlines
        majglx = inkex.Group.new("MajorXGridlines")
        yield majglx

        # Group for major y gridlines
        majgly = inkex.Group.new("MajorYGridlines")
        yield majgly

        # Group for major z gridlines
        majglz = inkex.Group.new("MajorZGridlines")
        yield majglz

        # Group for minor x gridlines
        if self.options.subdivs > 1:  # if there are any minor x gridlines
            minglx = inkex.Group.new("MinorXGridlines")
            yield minglx
        # Group for subminor x gridlines, if there are any minor minor x gridlines
        if self.options.subsubdivs > 1:
            mminglx = inkex.Group.new("SubMinorXGridlines")
            yield mminglx
        # Group for minor y gridlines, if there are any minor y gridlines
        if self.options.subdivs > 1:
            mingly = inkex.Group.new("MinorYGridlines")
            yield mingly
        # Group for subminor y gridlines, if there are any minor minor x gridlines
        if self.options.subsubdivs > 1:
            mmingly = inkex.Group.new("SubMinorYGridlines")
            yield mmingly
        # Group for minor z gridlines, if there are any minor y gridlines
        if self.options.subdivs > 1:
            minglz = inkex.Group.new("MinorZGridlines")
            yield minglz
        # Group for subminor z gridlines, if there are any minor minor x gridlines
        if self.options.subsubdivs > 1:
            mminglz = inkex.Group.new("SubMinorZGridlines")
            yield mminglz

        # Border of grid
        yield draw_rect(0, 0, xmax, ymax, self.options.border_th, "none", "Border")

        # X DIVISION
        # Shortcuts for divisions
        sd = self.options.subdivs
        ssd = self.options.subsubdivs

        # Initializing variable
        cpt_div = 0
        cpt_subdiv = 0
        cpt_subsubdiv = 0
        com_div = 0
        com_subdiv = 0
        com_subsubdiv = 0

        for i in range(1, (2 * self.options.x_divs * sd * ssd)):
            cpt_subsubdiv += 1
            com_subsubdiv = 1
            if cpt_subsubdiv == self.options.subsubdivs:
                cpt_subsubdiv = 0
                cpt_subdiv += 1
                com_subsubdiv = 0
                com_subdiv = 1
                com_div = 0

            if cpt_subdiv == self.options.subdivs:
                cpt_subdiv = 0
                com_subsubdiv = 0
                com_subdiv = 0
                com_div = 1

            if com_subsubdiv == 1:
                draw_line(
                    self.options.dx * i / sd / ssd,
                    0,
                    self.options.dx * i / sd / ssd,
                    ymax,
                    self.options.subsubdivs_th,
                    "MajorXDiv" + str(i),
                    mminglx,
                )
            if com_subdiv == 1:
                com_subdiv = 0
                draw_line(
                    self.options.dx * i / sd / ssd,
                    0,
                    self.options.dx * i / sd / ssd,
                    ymax,
                    self.options.subdivs_th,
                    "MajorXDiv" + str(i),
                    minglx,
                )
            if com_div == 1:
                com_div = 0
                draw_line(
                    self.options.dx * i / sd / ssd,
                    0,
                    self.options.dx * i / sd / ssd,
                    ymax,
                    self.options.divs_th,
                    "MajorXDiv" + str(i),
                    majglx,
                )

        # Y DIVISIONS
        # Shortcuts for divisions
        sd = self.options.subdivs
        ssd = self.options.subsubdivs

        taille = self.options.dx / sd / ssd  # Size of unity
        nb_ligne = (
            (self.options.x_divs + self.options.y_divs)
            * self.options.subdivs
            * self.options.subsubdivs
        )  # Global number of lines
        nb_ligne_x = (
            self.options.x_divs * self.options.subdivs * self.options.subsubdivs
        )  # Number of lines X
        nb_ligne_y = (
            self.options.y_divs * self.options.subdivs * self.options.subsubdivs
        )  # Number of lines Y

        # Initializing variable
        cpt_div = 0
        cpt_subdiv = 0
        cpt_subsubdiv = 0
        com_div = 0
        com_subdiv = 0
        com_subsubdiv = 0

        for l in range(1, int(nb_ligne * 4)):
            cpt_subsubdiv += 1
            com_subsubdiv = 1
            if cpt_subsubdiv == self.options.subsubdivs:
                cpt_subsubdiv = 0
                cpt_subdiv += 1
                com_subsubdiv = 0
                com_subdiv = 1
                com_div = 0

            if cpt_subdiv == self.options.subdivs:
                cpt_subdiv = 0
                com_subsubdiv = 0
                com_subdiv = 0
                com_div = 1

            if ((2 * l) - 1) < (2 * nb_ligne_x):
                txa = taille * ((2 * l) - 1)
                tya = ymax
                txb = 0
                tyb = ymax - taille / (2 * 0.866025) - (taille * (l - 1) / 0.866025)

                if com_subsubdiv == 1:
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.subsubdivs_th,
                        "MajorYDiv" + str(i),
                        mmingly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.subsubdivs_th,
                        "MajorZDiv" + str(l),
                        mminglz,
                    )
                if com_subdiv == 1:
                    com_subdiv = 0
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.subdivs_th,
                        "MajorYDiv" + str(i),
                        mingly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.subdivs_th,
                        "MajorZDiv" + str(l),
                        minglz,
                    )
                if com_div == 1:
                    com_div = 0
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.divs_th,
                        "MajorYDiv" + str(i),
                        majgly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.divs_th,
                        "MajorZDiv" + str(l),
                        majglz,
                    )

            if ((2 * l) - 1) == (2 * nb_ligne_x):
                txa = taille * ((2 * l) - 1)
                tya = ymax
                txb = 0
                tyb = ymax - taille / (2 * 0.866025) - (taille * (l - 1) / 0.866025)

                if com_subsubdiv == 1:
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.subsubdivs_th,
                        "MajorYDiv" + str(i),
                        mmingly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.subsubdivs_th,
                        "MajorZDiv" + str(l),
                        mminglz,
                    )
                if com_subdiv == 1:
                    com_subdiv = 0
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.subdivs_th,
                        "MajorYDiv" + str(i),
                        mingly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.subdivs_th,
                        "MajorZDiv" + str(l),
                        minglz,
                    )
                if com_div == 1:
                    com_div = 0
                    draw_line(
                        txa,
                        tya,
                        txb,
                        tyb,
                        self.options.divs_th,
                        "MajorYDiv" + str(i),
                        majgly,
                    )
                    draw_line(
                        xmax - txa,
                        tya,
                        xmax - txb,
                        tyb,
                        self.options.divs_th,
                        "MajorZDiv" + str(l),
                        majglz,
                    )

            if ((2 * l) - 1) > (2 * nb_ligne_x):
                txa = xmax
                tya = (
                    ymax
                    - taille / (2 * 0.866025)
                    - (taille * (l - 1 - ((2 * nb_ligne_x) / 2)) / 0.866025)
                )
                txb = 0
                tyb = ymax - taille / (2 * 0.866025) - (taille * (l - 1) / 0.866025)

                if tyb <= 0:
                    txa = xmax
                    tya = (
                        ymax
                        - taille / (2 * 0.866025)
                        - (taille * (l - 1 - ((2 * nb_ligne_x) / 2)) / 0.866025)
                    )
                    txb = taille * (2 * (l - (2 * nb_ligne_y)) - 1)
                    tyb = 0

                    if txb < xmax:
                        if com_subsubdiv == 1:
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.subsubdivs_th,
                                "MajorYDiv" + str(i),
                                mmingly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.subsubdivs_th,
                                "MajorZDiv" + str(l),
                                mminglz,
                            )
                        if com_subdiv == 1:
                            com_subdiv = 0
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.subdivs_th,
                                "MajorYDiv" + str(i),
                                mingly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.subdivs_th,
                                "MajorZDiv" + str(l),
                                minglz,
                            )
                        if com_div == 1:
                            com_div = 0
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.divs_th,
                                "MajorYDiv" + str(i),
                                majgly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.divs_th,
                                "MajorZDiv" + str(l),
                                majglz,
                            )

                else:
                    if txb < xmax:
                        if com_subsubdiv == 1:
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.subsubdivs_th,
                                "MajorYDiv" + str(i),
                                mmingly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.subsubdivs_th,
                                "MajorZDiv" + str(l),
                                mminglz,
                            )
                        if com_subdiv == 1:
                            com_subdiv = 0
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.subdivs_th,
                                "MajorYDiv" + str(i),
                                mingly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.subdivs_th,
                                "MajorZDiv" + str(l),
                                minglz,
                            )
                        if com_div == 1:
                            com_div = 0
                            draw_line(
                                txa,
                                tya,
                                txb,
                                tyb,
                                self.options.divs_th,
                                "MajorYDiv" + str(i),
                                majgly,
                            )
                            draw_line(
                                xmax - txa,
                                tya,
                                xmax - txb,
                                tyb,
                                self.options.divs_th,
                                "MajorZDiv" + str(l),
                                majglz,
                            )


if __name__ == "__main__":
    GridIsometric().run()



#!/usr/bin/env python

# Copyright (C) 2019 Gemy Cedric Activdesign.eu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
import inkwebeffect
import inkex
from inkex.localization import inkex_gettext as _


class InteractiveMockup(inkwebeffect.InkWebEffect):
    def add_arguments(self, pars):
        pars.add_argument(
            "--when", default="onclick", help="Event that will trigger the action"
        )
        pars.add_argument("--tab")

    def effect(self):
        self.ensureInkWebSupport()

        if len(self.options.ids) < 2:
            raise inkex.AbortExtension(
                _(
                    "You must select at least two elements. "
                    "The last one is the object you want to go to."
                )
            )

        el_from = list(self.svg.selection.values())[:-1]

        ev_code = "InkWeb.moveViewbox({from:this, to:'" + self.options.ids[-1] + "'})"
        for elem in el_from:
            prev_ev_code = elem.get(self.options.when)
            el_ev_code = ev_code + ";" + (prev_ev_code or "")
            elem.set(self.options.when, el_ev_code)


if __name__ == "__main__":
    InteractiveMockup().run()
